@@ -188,7 +188,7 @@ impl<'control> DebBuilder<'control> {
     ) -> Result<Self, DebError> {
         let entry = entry.into();
 
-        let data = entry.data.resolve()?;
+        let data = entry.resolve_content()?;
         let mut cursor = Cursor::new(&data);
         self.control_builder = self
             .control_builder
@@ -414,20 +414,8 @@ impl<'a> ControlTarBuilder<'a> {
         let control_data = control_buffer.into_inner()?;
 
         let mut manifest = self.extra_files.clone();
-        manifest.add_file_entry(
-            "control",
-            FileEntry {
-                data: control_data.into(),
-                executable: false,
-            },
-        )?;
-        manifest.add_file_entry(
-            "md5sums",
-            FileEntry {
-                data: self.md5sums.concat::<u8>().into(),
-                executable: false,
-            },
-        )?;
+        manifest.add_file_entry("control", control_data)?;
+        manifest.add_file_entry("md5sums", self.md5sums.concat::<u8>())?;
 
         write_deb_tar(writer, &manifest, self.mtime())
     }
@@ -461,11 +449,15 @@ pub fn write_deb_tar<W: Write>(
 
     // FileManifest is backed by a BTreeMap, so iteration is deterministic.
     for (rel_path, content) in files.iter_entries() {
-        let data = content.data.resolve()?;
+        let data = content.resolve_content()?;
 
         let mut header = new_tar_header(mtime)?;
         set_header_path(&mut builder, &mut header, rel_path, false)?;
-        header.set_mode(if content.executable { 0o755 } else { 0o644 });
+        header.set_mode(if content.is_executable() {
+            0o755
+        } else {
+            0o644
+        });
         header.set_size(data.len() as _);
         header.set_cksum();
         builder.append(&header, &*data)?;
@@ -496,13 +488,7 @@ mod tests {
 
         let builder = ControlTarBuilder::new(control)
             .set_mtime(Some(SystemTime::UNIX_EPOCH))
-            .add_extra_file(
-                "prerm",
-                FileEntry {
-                    data: vec![42].into(),
-                    executable: true,
-                },
-            )?
+            .add_extra_file("prerm", FileEntry::new_from_data(vec![42], true))?
             .add_data_file("usr/bin/myapp", &mut std::io::Cursor::new("data"))?;
 
         let mut buffer = vec![];
@@ -530,13 +516,7 @@ mod tests {
     #[test]
     fn test_write_data_tar_one_file() -> Result<()> {
         let mut manifest = FileManifest::default();
-        manifest.add_file_entry(
-            "foo/bar.txt",
-            FileEntry {
-                data: vec![42].into(),
-                executable: true,
-            },
-        )?;
+        manifest.add_file_entry("foo/bar.txt", FileEntry::new_from_data(vec![42], true))?;
 
         let mut buffer = vec![];
         write_deb_tar(&mut buffer, &manifest, 2)?;
@@ -565,13 +545,7 @@ mod tests {
 
         let mut manifest = FileManifest::default();
 
-        manifest.add_file_entry(
-            &long_path,
-            FileEntry {
-                data: vec![42].into(),
-                executable: false,
-            },
-        )?;
+        manifest.add_file_entry(&long_path, vec![42])?;
 
         let mut buffer = vec![];
         write_deb_tar(&mut buffer, &manifest, 2)?;
@@ -605,13 +579,7 @@ mod tests {
 
         let builder = DebBuilder::new(control)
             .set_compression(DebCompression::Zstandard(3))
-            .install_file(
-                "usr/bin/myapp",
-                FileEntry {
-                    data: vec![42].into(),
-                    executable: true,
-                },
-            )?;
+            .install_file("usr/bin/myapp", FileEntry::new_from_data(vec![42], true))?;
 
         let mut buffer = vec![];
         builder.write(&mut buffer)?;
